//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.RDF;

import ch.hearc.I04_05s08.GUI.Graph.*;
import ch.hearc.I04_05s08.Ontology.RDF.InstanceOfClass;
import java.awt.geom.*;
import java.awt.*;

/**
 *	A node for RDFS graph
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfResource extends RdfNode
{	
	/**
	 *	The ellipse (instead of a rectangle)
	 */
	private Ellipse2D ellipse;
	
	/**
	 *	Build a rdf node
	 *
	 *	@param ioc		the instance of class
	 */
	public RdfResource(InstanceOfClass ioc)
	{
		super(ioc);
		
		Dimension2D padding = getPadding();
		padding.setSize(getPadding().getWidth() + 20, getPadding().getHeight());
		setPadding(padding);
	}
	
	/**
	 *	Build a copy
	 *
	 *	@param rdfr		rdf resource node
	 */
	public RdfResource(RdfResource rdfr)
	{
		super(rdfr);
		
		ellipse = rdfr.ellipse;
		
		setPadding(rdfr.getPadding());
	}
	
	/**
	 *	Display a dialog box for this node
	 *
	 *	@param graph		the graph
	 */
	public void showDialog(GraphPanel graph)
	{
		super.showDialog(graph);
		
		if(graph instanceof RdfGraph)
		{
			new RdfDialog(this,(RdfGraph)graph);
		}
	}
	
	/**
	 *	Calculate the dimension of the node
	 *
	 *	@param g2d		graphic context
	 */
	public void calcDimension(Graphics2D g2d)
	{
		super.calcDimension(g2d);
		
		ellipse = new Ellipse2D.Double(
			getBounds().getX(),
			getBounds().getY(),
			getBounds().getWidth(),
			getBounds().getHeight()
		);
	}
	
	/**
	 *	Return a clone
	 *	
	 *	@return the cone
	 */
	public Object clone()
	{
		return (Object)new RdfResource(this);
	}
	
	/**
	 *	Draw the Node
	 *
	 *	@param g2d	Graphic context
	 */
	public void draw(Graphics2D g2d)
	{		
		calcDimension(g2d);
		
		// background
		g2d.setColor(getBackgroundColor());
		g2d.fill(ellipse);
		
		// foreground
		g2d.setColor(getForegroundColor());
		g2d.draw(ellipse);
		
		// text
		drawTitle(g2d);
	}
	
	/**
	 *	Test if the RDFS Property contains the point
	 *
	 *	@param p		the point
	 *	@return 		if the Property contains this point
	 */
	public boolean contains(Point2D p)
	{
		return ((ellipse != null)
			? ellipse.contains(p)
			: false);
	}
	
	/**
	 *	Get the point touching  the node between the givent point and the middle of the node
	 *
	 *	@param p	distant point
	 *	@param g2d	graphic context
	 *	@return the closest point of the point touching the node
	 */
	public Point2D getProximityPoint(Point2D p, Graphics2D g2d)
	{		
		calcDimension(g2d);
		
		Point2D gravity = getGravityPoint(g2d);
		
		double pl = (p.getY()-gravity.getY()) / (p.getX()-gravity.getX());
		
		double a = ellipse.getWidth() / 2.;
		double b = ellipse.getHeight() / 2.;
		
		double angle = Math.atan(pl);
		
		/*
			Many thanks to Antoine Lemoine and Daniel Yann.
		
			x^2/a^2 + y^2/b*2 = 1
		
			y = x tan(alpha)
		
			x^2/a^2 + x^2*tan(alpha)^2/b^2 = 1
		
			(b^2 + a^2*tan(alpha)^2)/a^2*b^2 = 1/x^2
			
			x = +/- (a*b)/sqrt(b^2 + a^2 tan(alpha)^2)
		
		*/
		double widthOffset = (a * b)/Math.sqrt(b*b + a*a*Math.pow(Math.tan(angle),2));
		double heightOffset = widthOffset * Math.tan(angle);
		
		if(gravity.getX() > p.getX())
		{
			widthOffset = -widthOffset;
			heightOffset = -heightOffset;
		}
		
		return new Point2D.Double(
			gravity.getX() + widthOffset,
			gravity.getY() + heightOffset
		);
	}
}
