//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDF;

import ch.hearc.I04_05s08.Ontology.RDFS.RdfsResource;

/**
 *	A RDF instance that represent a RDFS Resource
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public abstract class Instance
{
	/**
	 *	The resource
	 */
	private RdfsResource resource;
	
	/**
	 *	The instance's name
	 */
	private String name;
	
	/**
	 *	Build the instance
	 *
	 *	@param resource		a RDFS Resource
	 *	@param name			the name
	 */
	public Instance(RdfsResource resource, String name)
	{
		this.resource = resource;
		this.name = name;
	}
	
	/**
	 *	Build the instance
	 *
	 *	@param name		the name
	 */
	public Instance(String name)
	{
		this(null, name);
	}
	
	/**
	 *	Build a copy
	 *
	 *	@param instance		the origin instance
	 */
	public Instance(Instance instance)
	{
		this(instance.name);
		if(instance.isInstance())
			resource = (RdfsResource)instance.resource.clone();
	}
	
	/**
	 *	Get the resource that is instanciated
	 *
	 *	@return the resource
	 */
	public RdfsResource getResource()
	{
		return resource;
	}
	
	/**
	 *	Set the resource
	 *
	 *	@param resource		the resource
	 */
	public void setResource(RdfsResource resource)
	{
		this.resource = resource;
	}
	
	/**
	 *	Get the name of the resource
	 *
	 *	@return the name of the resource
	 */
	public String getName()
	{
		return (isInstance()
			? resource.getName()
			: name);
	}
	
	/**
	 *	Get the short name
	 *
	 *	@return	the short name of the Resource
	 */
	public String getShortName()
	{
		String[] res;
		res = getName().split("#");
		if(res.length == 1)
		{
			res = res[0].split("/");
		}
		
		return (res.length >= 1) ? res[res.length-1] : name;
	}
	
	/**
	 *	Get the name of the resource
	 *
	 *	@param name			the name of the resource
	 */
	public boolean setName(String name)
	{
		this.name = name;
		return true;
	}
	
	/**
	 *	Get the value of the resource
	 *
	 *	@return the value of the resource
	 */
	public String getValue()
	{
		return name;
	}
	
	/**
	 *	Get the short name
	 *
	 *	@return	the short name of the Resource
	 */
	public String getShortValue()
	{
		String[] res;
		res = getValue().split("#");
		if(res.length == 1)
		{
			res = res[0].split("/");
		}
		
		return (res.length >= 1) ? res[res.length-1] : name;
	}
	
	/**
	 *	Set the value of the resource
	 *
	 *	@param value		the value	
	 */
	public void setValue(String value)
	{
		this.name = value;
	}
	
	/**
	 *	Get information about if the instance has a resource
	 *
	 *	@return	if this instance have a parent resource
	 */
	public boolean isInstance()
	{
		return (resource != null);
	}
}