//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDF;

import ch.hearc.I04_05s08.Ontology.RDFS.RdfsClass;

/**
 *	A RDF Resource that represent a property of a Class
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class InstanceOfClass extends Instance
{	
	/**
	 *	URI of the XMLS#String
	 */
	public static final String literal = "http://www.w3.org/2000/10/XMLSchema#String";
	
	/**
	 *	The literal value
	 */
	private String value;
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 *	Construct the instance of the class
	 *
	 *	@param c		that class that is be instanciated
	 */
	public InstanceOfClass(RdfsClass c, String name)
	{
		super(c,name);
	}
	
	/**
	 *	Construct the instance of an unexisting class
	 *
	 *	@param classname	the name of the class
	 */
	public InstanceOfClass(String classname)
	{
		super(classname);
	}
	
	/**
	 *	Construct a literal class
	 */
	public InstanceOfClass()
	{
		super("http://www.w3.org/2000/10/XMLSchema#String");
	}
	
	/**
	 *	Construct a copy
	 */
	public InstanceOfClass(InstanceOfClass ioc)
	{
		super(ioc);
		value = ioc.value;
	}
	
	// Public //////////////////////////////////////////////////////////////////////
	
	/**
	 *	Build a clone
	 *
	 *	@return a clone
	 */
	public Object clone()
	{
		return (Object)new InstanceOfClass(this);
	}
	
	/**
	 *	String representation
	 *
	 *	@return string representation
	 */
	public String toString()
	{
		String str = "";
		if(!isLiteral())
		{
			if(isInstance())
			{
				str += "(rdfs) ";
			}
			str += getName() + " " + getValue();
		}
		else
		{
			str = "(xmls:literal) " + getValue();
		}
		return str;
	}
	
	/**
	 *	Apply the known class to instance
	 *
	 *	@param c			the class
	 *	@return if it pass
	 */
	public boolean setClass(RdfsClass c)
	{
		if(!isInstance())
		{
			setResource(c);
			return true;
		}
		else
		{
			return false;
		}
	}	
	
	/**
	 *	Verify is the class is another classes (multiple heritance)
	 *
	 *	@param tuple		parent classes
	 *	@return			if this class <strong>is</strong> the tuple
	 */
	public boolean isA(RdfsClass[] tuple)
	{
		if(isInstance())
		{
			return ((RdfsClass)getResource()).isA(tuple);
		}
		else
		{
			return true;
		}
	}
	
	/**
	 *	Verify if the property is literal
	 *
	 *	@return literal or not ?
	 */
	public boolean isLiteral()
	{
		return(getName().equals(InstanceOfClass.literal));
	}
	
	/**
	 * Set the value of the class
	 *
	 * @param value	the value that would be applied to Property
	 * @return 		if the value could be applied
	 */
	public void setValue(String value)
	{
		if(isLiteral())
			this.value = value;
		else
			super.setValue(value);
	}
	
	/**
	 * Get the value of the class
	 *
	 * @return 		the property's value
	 */
	public String getValue()
	{
		if(isLiteral())
			return value;
		else
			return super.getValue();
	}
}
