//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDFS;

import java.util.LinkedList;
import java.util.Iterator;

/**
 *	A representation of a rdfs:Class
 *
 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_class">w3c</a>
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfsClass extends RdfsResource
{
	/**
	 *  Classes from this is extended
	 *
	 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_subclassof">w3c</a>
	 */
	private LinkedList subClassOf;
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 *	Minimal constructor
	 */
	public RdfsClass(String name)
	{
		this(name,null);
	}
	
	
	/**
	 *	Minimal recommended constructor
	 */
	public RdfsClass(String name, String label)
	{
		this(name,label,null);
	}
	
	
	/**
	 *	Full constructor
	 */
	public RdfsClass(String name, String label, String comment)
	{
		super(name,label,comment);
		
		subClassOf = new LinkedList();
		
		init();
	}
	
	/**
	 *	A new class from another one
	 *
	 *	@param c	Classe source
	 */
	public RdfsClass(RdfsClass c)
	{
		super((RdfsResource)c);
		
		// copy every list
		subClassOf = copyList(c.subClassOf);
	}
	
	/**
	 *	Build the objects of Class
	 */
	private void init()
	{
		subClassOf = new LinkedList();
	}
	
	// Public /////////////////////////////////////////////////////////////////
	
	/**
	 *	String representation of the class
	 *
	 *	@return			class and subclasses
	 */
	public String toString()
	{
		String str = super.toString();
		
		for(Iterator i=subClassOf.iterator(); i.hasNext();)
		{
			str += "\n\tsubClass : "+((RdfsClass)(i.next())).getName();
		}
		
		return str;
	}
	
	/**
	 * Duplicate the current Class
	 *
	 * @return copy of the Class
	 */
	public Object clone()
	{
		return (Object)new RdfsClass(this);
	}

	/**
	 * Return the subClass of the Class is
	 *
	 * @return the subClass
	 */
	public RdfsClass[] getSupClasses()
	{
		RdfsClass[] subclasses;
		Iterator i;
		
		subclasses = new RdfsClass[subClassOf.size()];
		
		i = subClassOf.iterator();
		int j=0;
		while(i.hasNext())
		{
			subclasses[j] = (RdfsClass)i.next();
			j++;
		}
		
		return subclasses;
	}
	
	/**
	 *	Add a superClass
	 *
	 *	@param c	new subClass
	 */
	public boolean addSupClass(RdfsClass c)
	{
		return (!subClassOf.contains(c)
			? subClassOf.add((Object)c)
			: false);
	}
	
	/**
	 *	Dereference a superClass
	 *
	 *	@param c	subClass that will be dereferenced
	 */
	public boolean delSupClass(RdfsClass c)
	{
		return subClassOf.remove((Object)c);
	}
	
	/**
	 *	Verify is the class is another classes (multiple heritance)
	 *
	 *	@param tuple		parent classes
	 *	@return			if this class <strong>is</strong> the tuple
	 */
	public boolean isA(RdfsClass[] tuple)
	{
		Iterator i,j;
		boolean is = false;
		
		// recursive test
		i = subClassOf.iterator();
		while(i.hasNext())
		{
			is |= ((RdfsClass)(i.next())).isA(tuple);
		}
		
		if(is)
			return is;
		
		// if heritance is single the test is easy
		if(tuple.length == 1)
		{
			is = equals(tuple[0]);
			is |= (tuple[0].getName().equals("http://www.w3.org/2000/01/rdf-schema#Resource"));
		}
		// multiple heritance
		// double
		else if(tuple.length == 2)
		{			
			i = subClassOf.iterator();
			while(i.hasNext())
			{
				RdfsClass c = (RdfsClass)(i.next());
				j = subClassOf.iterator();
				
				while(j.hasNext())
				{
					RdfsClass d = (RdfsClass)(j.next());
					is |= c.isA(new RdfsClass[]{tuple[0]})
						&& d.isA(new RdfsClass[]{tuple[1]});
				}
				
				if(is)
					return is;
			}
		}
		// triple
		// too complex for now
		else if(tuple.length > 2)
		{
			is = true;
			System.err.println("multiple heritance more than 2");
		}
		
		return is;
	}
}
