//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDFS;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.rdql.*;
import com.hp.hpl.jena.vocabulary.*;

import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.ByteArrayOutputStream;
import java.util.Iterator;

/**
 *	A manager for RDF Schema
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfsManager
{
	private static RdfsProperty XString = new RdfsProperty("http://www.w3.org/2000/10/XMLSchema#string");
	
	/**
	 *	Save a rdfs file
	 *
	 *	@param filename		path of the source file
	 *	@param file			the rdfs file
	 */
	public static boolean saveRdfs(String filename, RdfsFile file)
	{
		OntModel model = buildModel(file);
		
		try
		{
			model.write(new PrintWriter(
				new OutputStreamWriter(
					new FileOutputStream(filename)
				)
			));
		}
		catch (Exception e)
		{
			System.out.println("error : "+e);
		}
		
		return true;
	}
	
	/**
	 *	Save a rdfs file into a string
	 *
	 *	@param file			the rdfs file
	 */
	public static String rdfsToString(RdfsFile file)
	{
		OntModel model = buildModel(file);
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		
		try
		{
			model.write(new PrintWriter(baos));
		}
		catch (Exception e)
		{
			System.out.println("error : "+e);
		}
		
		return baos.toString();
	}
	
	/**
	 *	Build a model with the RdfsFile
	 *
	 *	@param	file	the rdfs file
	 *	@return	the model
	 */
	public static OntModel buildModel(RdfsFile file)
	{
		OntModel model = ModelFactory.createOntologyModel(OntModelSpec.RDFS_MEM,null);
		
		// classes
		RdfsClass[] classes = file.getClasses();
		
		OntClass[] ontclasses = new OntClass[classes.length];
		for(int i=0;i<classes.length;++i)
		{
			ontclasses[i] = model.createClass(classes[i].getName());
		}
		
		// properties
		RdfsProperty[] properties = file.getProperties();
		
		OntProperty[] ontprops = new OntProperty[properties.length];
		for(int i=0;i<properties.length;++i)
		{
			ontprops[i] = model.createOntProperty(properties[i].getName());
		}
		
		// classes
		for(int i=0; i<classes.length;++i)
		{
			RdfsClass[] sub = classes[i].getSupClasses();
			for(int j=0; j<sub.length; ++j)
			{
				getOntClass(ontclasses, sub[j].getName()).addSubClass(ontclasses[i]);
			}
			
			String comment = classes[i].getComment();
			String label = classes[i].getLabel();
			
			if(comment != null)
			{
				ontclasses[i].addComment(comment, null);
			}
			if(label != null)
			{
				ontclasses[i].addLabel(label, null);
			}
		}
		
		// properties
		for(int i=0; i<properties.length;++i)
		{
			RdfsProperty[] sub = properties[i].getSupProperties();
			for(int j=0; j<sub.length; ++j)
			{
				getOntProperty(ontprops, sub[j].getName()).addSubProperty(ontprops[i]);
			}
			
			RdfsClass[] range = properties[i].getRange();
			for(int j=0; j<range.length; ++j)
			{
				if(!range[j].getName().equals("http://www.w3.org/2000/10/XMLSchema#String"))
					ontprops[i].addRange(getOntClass(ontclasses, range[j].getName()));
				else
					ontprops[i].addRange(model.createClass(range[j].getName()));
			}
			
			RdfsClass[] domain = properties[i].getDomain();
			for(int j=0; j<domain.length; ++j)
			{
				OntClass domainc = getOntClass(ontclasses, domain[j].getName());
				if(domainc != null)
					ontprops[i].addDomain(domainc);
			}
			
			String comment = properties[i].getComment();
			String label = properties[i].getLabel();
			
			if(comment != null)
			{
				ontprops[i].addComment(comment, null);
			}
			if(label != null)
			{
				ontprops[i].addLabel(label, null);
			}
		}
		
		return model;
	}
	
	/**
	 *	Get a resource from its uri
	 *
	 *	@param resources		the ontResources
	 *	@param uri				uri that searched
	 *	@return					the OntRessource with the uri
	 */
	private static OntResource getResource(Resource[] resources, String uri)
	{
		OntResource find = null;
		for(int i=0;i<resources.length;++i)
		{
			if(resources[i].getURI().equals(uri))
			{
				find = (OntResource)resources[i];
				break;
			}
		}
		return find;
	}
	
	/**
	 *	Get a class from its uri
	 *
	 *	@param classes		the ontClasses
	 *	@param uri			uri that searched
	 *	@return				the OntClass with the uri
	 */
	private static OntClass getOntClass(OntResource[] classes, String uri)
	{
		return (OntClass)getResource(classes, uri);
	}
	
	/**
	 *	Get a property from its uri
	 *
	 *	@param props		the ontProperties
	 *	@param uri		uri that searched
	 *	@return			the OntProperty with the uri
	 */
	private static OntProperty getOntProperty(OntResource[] props, String uri)
	{
		return (OntProperty)getResource(props, uri);
	}
	
	// -- //////////////////////////////////////////////////////////////////////////
	
	/**
	 *	Build a rdfs file
	 *
	 *	@param filename		path of the source file
	 *	@return				the rdfs file
	 */
	public static RdfsFile openRdfs(String filename)
	{
		RdfsFile file = new RdfsFile(filename);
		
		// model's loading
		Model model = ModelFactory.createDefaultModel();
		model.read(file.getReader(),"");
		file.close();
		
		// parsing //
		
		// Classes 
		String req = "SELECT ?x WHERE (?x  rdf:type rdfs:Class)";
		
		QueryResults qr = runQuery(model, req);
		for (Iterator i = qr; i.hasNext();)
		{
			ResultBinding res = (ResultBinding)i.next();
			
			RdfsClass c = buildClass(file, res.getValue("x").toString());
			
			// subClassOf
			String r = "SELECT ?x WHERE (<"+c.getName()+"> rdfs:subClassOf ?x)";

			QueryResults qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				RdfsClass sub = buildClass(file, rb.getValue("x").toString());
				
				c.addSupClass(sub);
			}
			qre.close();
			
			// label
			r = "SELECT ?x WHERE (<"+c.getName()+"> rdfs:label ?x)";

			qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				
				c.setLabel(rb.getValue("x").toString());
			}
			qre.close();
			
			// comment
			r = "SELECT ?x WHERE (<"+c.getName()+"> rdfs:comment ?x)";

			qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				
				c.setComment(rb.getValue("x").toString());
			}
			qre.close();
		}
		qr.close();
		
		// Properties
		req = "SELECT ?x WHERE (?x  rdf:type rdf:Property)";
		
		qr = runQuery(model, req);
		for (Iterator i = qr; i.hasNext();)
		{
			ResultBinding res = (ResultBinding)i.next();
			RdfsProperty p = buildProperty(file, res.getValue("x").toString());
			
			// subPropertyOf
			String r = "SELECT ?x WHERE (<"+p.getName()+"> rdfs:subPropertyOf ?x)";
			
			QueryResults qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				RdfsProperty sub = buildProperty(file, rb.getValue("x").toString());
				
				p.addSupProperty(sub);
			}
			qre.close();
			
			// range
			r = "SELECT ?x WHERE (<"+p.getName()+"> rdfs:range ?x)";
			
			qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				RdfsClass range = buildClass(file, rb.getValue("x").toString());
				
				p.addRange(range);
			}
			qre.close();
			
			// domain
			r = "SELECT ?x WHERE (<"+p.getName()+"> rdfs:domain ?x)";
			
			qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				RdfsClass domain = buildClass(file, rb.getValue("x").toString());
				
				p.addDomain(domain);
			}
			qre.close();
			
			// label
			r = "SELECT ?x WHERE (<"+p.getName()+"> rdfs:label ?x)";

			qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				
				p.setLabel(rb.getValue("x").toString());
			}
			qre.close();
			
			// comment
			r = "SELECT ?x WHERE (<"+p.getName()+"> rdfs:comment ?x)";

			qre = runQuery(model, r);
			for(Iterator j = qre; j.hasNext();)
			{
				ResultBinding rb = (ResultBinding)j.next();
				
				p.setComment(rb.getValue("x").toString());
			}
			qre.close();
		}
		qr.close();
		
		return file;
	}
	
	/**
	 *	Execute a RDQL query
	 *
	 *	@param model		current model
	 * 	@param query		RDQL query
	 */
	private static QueryResults runQuery(Model model, String query)
	{
		Query q = new Query(query);
		
		q.setSource(model);
		
		QueryExecution qe = new QueryEngine(q);
		
		return qe.exec();
	}
	
	/**
	 *	Return a class from the rdfs file
	 *
	 *	@param file		rdfs file
	 *	@param name		identifier of the class (his url)
	 */
	private static RdfsClass buildClass(RdfsFile file, String name)
	{
		RdfsClass c = file.getClass(name);
		if(c != null)
		{
			return c;
		}
		else
		{
			c = new RdfsClass(name);
			if(!name.equals("http://www.w3.org/2000/10/XMLSchema#String"))
				file.add(c);
			return c;
		}
	}
	
	/**
	 *	Return a property from the rdfs file
	 *
	 *	@param file		rdfs file
	 *	@param name		identifier of the property (his url)
	 */
	private static RdfsProperty buildProperty(RdfsFile file, String name)
	{
		RdfsProperty p = file.getProperty(name);
		if(p != null)
		{
			return p;
		}
		else
		{
			p = new RdfsProperty(name);
			file.add(p);
			return p;
		}
	}
	
	// --
	
	/**
	 *	Get a class from some RDFS
	 *
	 *	@param rdfs		RDF Schema
	 *	@param name		searched class
	 */
	public static RdfsClass getClass(RdfsFile[] rdfs, String name)
	{
		RdfsClass c = null;
		for(int i=0; i<rdfs.length; ++i)
		{
			c = rdfs[i].getClass(name);
			
			if(c != null)
			{
				break;
			}
		}
		return c;
	}
	
	/**
	 *	Get a property from some RDFS
	 *
	 *	@param rdfs		RDF Schema
	 *	@param name		searched property
	 */
	public static RdfsProperty getProperty(RdfsFile[] rdfs, String name)
	{
		RdfsProperty p = null;
		for(int i=0; i<rdfs.length; ++i)
		{
			p = rdfs[i].getProperty(name);
			if(p != null)
			{
				break;
			}
		}
		return p;
	}
}
