//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDFS;

import java.util.LinkedList;
import java.util.Iterator;

/**
 *	A Property for a rdfs:class
 *
 *
 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_property">w3c</a>
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfsProperty extends RdfsResource
{
	/**
	 *	Classes that the property is applied
	 *
	 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_domain">w3c</a>
	 */
	private LinkedList domain;
		
	/**
	 *	Classes that the property can contain
	 *
	 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_range">w3c</a>
	 */
	private LinkedList range;
		
	/**
	 *  Properties from this is extended
	 *
	 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_subpropertyof">w3c</a>
	 */
	private LinkedList subPropertyOf;
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 *	Minimal constructor
	 */
	public RdfsProperty(String name)
	{
		super(name);
		
		init();
	}
	
	
	/**
	 *	Minimal recommended constructor
	 */
	public RdfsProperty(String name, String label)
	{
		super(name,label);
		
		init();
	}
	
	
	/**
	 *	Full constructor
	 */
	public RdfsProperty(String name, String label, String comment)
	{
		super(name,label,comment);
		
		init();
	}
	
	/**
	 *	A new property from another one
	 *
	 *	@param p	Property source
	 */
	public RdfsProperty(RdfsProperty p)
	{
		super((RdfsResource)p);
		
		// copy every list
		domain = copyList(p.domain);
		range = copyList(p.range);
		subPropertyOf = copyList(p.subPropertyOf);
	}
	
	/**
	 *	Clone this property
	 *
	 *	@return a clone of this
	 */
	public Object clone()
	{
		return new RdfsProperty(this);
	}
	
	/**
	 *	Build the object of Property
	 */
	private void init()
	{
		domain = new LinkedList();
		range = new LinkedList();
		subPropertyOf = new LinkedList();
	}
	
	// Public /////////////////////////////////////////////////////////////////
	
	/**
	 *	String representation of the property
	 *
	 *	@return			property, subproperties, domains and ranges
	 */
	public String toString()
	{
		String str = super.toString();
		
		for(Iterator i=subPropertyOf.iterator(); i.hasNext();)
		{
			str += "\n\tsubProp : "+((RdfsProperty)(i.next())).getName();
		}
		for(Iterator i=domain.iterator(); i.hasNext();)
		{
			str += "\n\tdomain : "+((RdfsClass)(i.next())).getName();
		}
		for(Iterator i=range.iterator(); i.hasNext();)
		{
			str += "\n\trange : "+((RdfsClass)(i.next())).getName();
		}
		
		return str;
	}
	
	/**
	 * Return the domain of the Property
	 *
	 * @return the domain of the Property
	 */
	public RdfsClass[] getDomain()
	{		
		if(domain.size() > 0)
		{
			RdfsClass[] domains;
			Iterator i;
			
			// sub domains
			RdfsClass subDomains[][] = new RdfsClass[subPropertyOf.size()][];
			int c = 0;
			int size = 0;
			
			for(i = subPropertyOf.iterator(); i.hasNext();)
			{
				subDomains[c] = ((RdfsProperty)i.next()).getDomain();
				size += subDomains[c].length;
				c++;
			}
			
			domains = new RdfsClass[size + domain.size()];
			int j=0;
			
			for(int k=0; k<subDomains.length; k++)
			{
				for(int l=0; l<subDomains[k].length; l++)
				{
					domains[j] = subDomains[k][l];
					j++;
				}
			}
			
			// local domains
			
			i = domain.iterator();
			while(i.hasNext())
			{
				domains[j] = (RdfsClass)i.next();
				j++;
			}
			
			return domains;
		}
		else
		{
			return new RdfsClass[]{new RdfsClass("http://www.w3.org/2000/01/rdf-schema#Resource")};
		}
	}
	
	/**
	 *	Add a domain
	 *
	 *	@param d	new domain
	 */
	public boolean addDomain(RdfsClass d)
	{
		return ((!domain.contains(d))
			? domain.add((Object)d)
			: false);
	}
	
	/**
	 *	Remove a specified domain from the property
	 *
	 *	@param d	domain that will be removed
	 */
	public boolean delDomain(RdfsClass d)
	{
		return domain.remove((Object)d);
	}

	/**
	 * Return the range of the Property
	 *
	 * @return the range of the Property
	 */
	public RdfsClass[] getRange()
	{
		if(range.size() > 0)
		{
			RdfsClass[] ranges;
			Iterator i;
			
			// sub ranges
			RdfsClass subRanges[][] = new RdfsClass[subPropertyOf.size()][];
			int c = 0;
			int size = 0;
			
			for(i = subPropertyOf.iterator(); i.hasNext();)
			{
				subRanges[c] = ((RdfsProperty)i.next()).getRange();
				size += subRanges[c].length;
				c++;
			}
			
			ranges = new RdfsClass[range.size() + size];
			
			int j=0;
			
			for(int k=0; k<subRanges.length; k++)
			{
				for(int l=0; l<subRanges[k].length; l++)
				{
					ranges[j] = subRanges[k][l];
					j++;
				}
			}
			
			i = range.iterator();
			//int j=0;
			while(i.hasNext())
			{
				ranges[j] = (RdfsClass)i.next();
				j++;
			}
			
			return ranges;
		}
		else
		{
			return new RdfsClass[]{new RdfsClass("http://www.w3.org/2000/10/XMLSchema#String")};
		}
	}
	
	/**
	 *	Add a range
	 *
	 *	@param r	new range
	 */
	public boolean addRange(RdfsClass r)
	{
		return (!range.contains(r) && !r.getName().equals("http://www.w3.org/2000/10/XMLSchema#String")
			? range.add((Object)r)
			: false);
	}
	
	/**
	 *	Remove a specified range from the property
	 *
	 *	@param r	range that will be removed
	 */
	public boolean delRange(RdfsClass r)
	{
		return range.remove((Object)r);
	}
	
	/**
	 * Return the subProperty of the Property is
	 *
	 * @return the subProperty
	 */
	public RdfsProperty[] getSupProperties()
	{
		RdfsProperty[] subproperties;
		Iterator i;
		
		subproperties = new RdfsProperty[subPropertyOf.size()];
		
		i = subPropertyOf.iterator();
		int j=0;
		while(i.hasNext())
		{
			subproperties[j] = (RdfsProperty)i.next();
			j++;
		}
		
		return subproperties;
	}
	
	/**
	 *	Add a superProperty
	 *
	 *	@param p	new subProperty
	 */
	public boolean addSupProperty(RdfsProperty p)
	{
		return (!subPropertyOf.contains(p)
			? subPropertyOf.add((Object)p)
			: false);
	}
	
	/**
	 *	Dereference a superProperty
	 *
	 *	@param p	subProperty that will be dereferenced
	 */
	public boolean delSupProperty(RdfsProperty p)
	{
		return subPropertyOf.remove((Object)p);
	}
}
